// Copyright 1998-2005, 2010-2012 Omni Development, Inc. All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// <http://www.omnigroup.com/developer/sourcecode/sourcelicense/>.

#import "OAFileWell.h"

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <OmniBase/OmniBase.h>

#import "OADragController.h"
#import "OAPasteboardHelper.h"

RCS_ID("$Id$")

@implementation OAFileWell

- (NSImage *)imageForFiles;
{
    if (![files count])
        return nil;

    return [[NSWorkspace sharedWorkspace] iconForFiles:files];
}

- (void)drawRect:(NSRect)aRect;
{
    NSImage *image = [self imageForFiles];
    if (image == nil)
        return;

    NSSize imageSize = [image size];
    NSRect bounds = self.bounds;
    NSPoint imagePoint;
    imagePoint.x = bounds.origin.x + (bounds.size.width - imageSize.width) / 2.0f;
    imagePoint.y = bounds.origin.y + (bounds.size.height - imageSize.height) / 2.0f;
    [image compositeToPoint:imagePoint operation:NSCompositeSourceOver];
}

- (BOOL)acceptsIncomingDrags;
{
    return acceptIncomingDrags;
}

- (NSArray *)files;
{
    return files;
}

- (void)setAcceptIncomingDrags:(BOOL)acceptIncoming;
{
    // UNDONE: not implemented to accept incoming yet
}

- (void)setFiles:(NSArray *)someFiles;
{
    [files autorelease];
    files = [someFiles retain];
    [self setNeedsDisplay:YES];
}

- (void)mouseDown:(NSEvent *)event;
{
    OAPasteboardHelper *helper;
    NSImage *dragImage;
    NSPoint where;

    if (!(dragImage = [self imageForFiles]))
        return;

    if ([event clickCount] > 1) {
        NSWorkspace *workspace;
        NSEnumerator *enumerator;
        NSString *path;

        workspace = [NSWorkspace sharedWorkspace];
        enumerator = [files objectEnumerator];
        while((path = [enumerator nextObject]))
            [workspace openFile:path fromImage:dragImage at:NSZeroPoint inView:self];
    }

    where = [self convertPoint:[event locationInWindow] fromView:nil];
    helper = [OAPasteboardHelper helperWithPasteboardNamed:NSDragPboard];
    [helper declareTypes:[NSArray arrayWithObject:NSFilenamesPboardType] owner:self];
    [[OADragController sharedDragController] startDragFromView:self image:dragImage atPoint:where offset:NSZeroPoint event:event slideBack:NO pasteboardHelper:helper];
}

- (void)pasteboard:(NSPasteboard *)pasteboard provideDataForType:(NSString *)type;
{
    [pasteboard setPropertyList:files forType:type];
}

@end
